/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package fr.amap.lidar.amapvox.gui.task;

import fr.amap.commons.util.ProcessingAdapter;
import fr.amap.lidar.amapvox.commons.Configuration;
import fr.amap.lidar.amapvox.simulation.hemi.HemiScanView;
import fr.amap.lidar.amapvox.simulation.transmittance.TransmittanceSim;
import fr.amap.lidar.amapvox.simulation.transmittance.lai2xxx.Lai2xxxSim;
import fr.amap.lidar.amapvox.util.AVoxTask;
import fr.amap.lidar.amapvox.voxelisation.als.AlsShotVoxelisation;
import fr.amap.lidar.amapvox.voxelisation.als.LasVoxelisation;
import fr.amap.lidar.amapvox.voxelisation.configuration.ALSVoxCfg;
import fr.amap.lidar.amapvox.voxelisation.configuration.TLSVoxCfg;
import fr.amap.lidar.amapvox.voxelisation.postproc.ButterflyRemover;
import fr.amap.lidar.amapvox.voxelisation.postproc.VoxelFileMerging;
import fr.amap.lidar.amapvox.voxelisation.tls.PTGVoxelisation;
import fr.amap.lidar.amapvox.voxelisation.tls.PTXVoxelisation;
import fr.amap.lidar.amapvox.voxelisation.tls.RSPVoxelisation;
import fr.amap.lidar.amapvox.voxelisation.tls.RxpVoxelisation;
import java.io.File;
import java.io.IOException;
import javafx.concurrent.Service;
import javafx.concurrent.Task;
import javafx.scene.image.Image;
import org.jdom2.JDOMException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 *
 * @author calcul
 */
public class ServiceProvider {

    private final File file;
    private final int ncpu;
    private final String type;
    final Logger LOGGER = LoggerFactory.getLogger(ServiceProvider.class);

    public final static Image VOXELIZATION_IMG = new Image(ServiceProvider.class.getResourceAsStream("/fxml/icons/voxels.png"));
    public final static Image TRANSMITTANCE_IMG = new Image(ServiceProvider.class.getResourceAsStream("/fxml/icons/sun.png"));
    public final static Image HEMI_IMG = new Image(ServiceProvider.class.getResourceAsStream("/fxml/icons/hemispherical.png"));
    public final static Image CANOPEE_ANALYZER_IMG = new Image(ServiceProvider.class.getResourceAsStream("/fxml/icons/lai2200.png"));
    public final static Image MISC_IMG = new Image(ServiceProvider.class.getResourceAsStream("/fxml/icons/configure.png"));

    public ServiceProvider(File file, int ncpu) {
        this.file = file;
        this.ncpu = ncpu;
        String strType = null;
        try {
            strType = Configuration.readType(file);
        } catch (JDOMException | IOException ex) {
            LOGGER.error("Undefined type of configuration file", ex);

        }
        this.type = strType;
    }

    public ServiceProvider(File file) {
        this(file, 1);
    }

    public Image icon() {

        switch (type) {
            case "voxelisation-ALS":
            case "voxelisation-TLS":
                return VOXELIZATION_IMG;
            case "transmittance":
                return TRANSMITTANCE_IMG;
            case "LAI2000":
            case "LAI2200":
                return CANOPEE_ANALYZER_IMG;
            case "Hemi-Photo":
                return HEMI_IMG;
        }

        return MISC_IMG;
    }

    public Service provide() {

        switch (type) {
            case "voxelisation-ALS":
                final ALSVoxCfg alsVoxCfg = new ALSVoxCfg();
                try {
                    alsVoxCfg.readConfiguration(file);
                } catch (Exception ex) {
                    LOGGER.error("Failed to read configuration file " + file.getAbsolutePath(), ex);
                    return null;
                }
                switch (alsVoxCfg.getInputType()) {
                    case LAS_FILE:
                    case LAZ_FILE:
                        return createService(LasVoxelisation.class, file, ncpu);
                    case SHOTS_FILE:
                        return createService(AlsShotVoxelisation.class, file, 1);
                }
            case "voxelisation-TLS":
                final TLSVoxCfg tLSVoxCfg = new TLSVoxCfg();
                try {
                    tLSVoxCfg.readConfiguration(file);
                } catch (Exception ex) {
                    LOGGER.error("Failed to read configuration file " + file.getAbsolutePath(), ex);
                    return null;
                }
                switch (tLSVoxCfg.getInputType()) {
                    case RSP_PROJECT:
                        return createService(RSPVoxelisation.class, file, ncpu);
                    case RXP_SCAN:
                        return createService(RxpVoxelisation.class, file, ncpu);
                    case PTG_PROJECT:
                        return createService(PTGVoxelisation.class, file, ncpu);
                    case PTX_PROJECT:
                        return createService(PTXVoxelisation.class, file, ncpu);
                }
            case "transmittance":
                return createService(TransmittanceSim.class, file, 1);
            case "LAI2000":
            case "LAI2200":
                return createService(Lai2xxxSim.class, file, 1);
            case "Hemi-Photo":
                return createService(HemiScanView.class, file, 1);
            case "merging":
                return createService(VoxelFileMerging.class, file, 1);
            case "butterfly-removing":
                return createService(ButterflyRemover.class, file, 1);
        }

        LOGGER.error("Undefined type of configuration file: " + type);
        return null;
    }

    private Service<File[]> createService(Class className, File file, int ncpu) {
        return new AvoxService(className, file, ncpu);
    }

    class AvoxService extends Service<File[]> {

        private AVoxTask task;
        private final Class className;
        private final File file;
        private final int ncpu;

        AvoxService(Class className, File file, int ncpu) {
            this.className = className;
            this.file = file;
            this.ncpu = ncpu;
        }

        @Override
        protected Task<File[]> createTask() {
            return new Task() {
                @Override
                protected File[] call() throws Exception {

                    task = (AVoxTask) className.getConstructor(File.class, int.class).newInstance(file, ncpu);
                    updateMessage(task.getName() + " started...");
                    updateTitle(task.getName());
                    task.addProcessingListener(new ProcessingAdapter() {
                        @Override
                        public void processingStepProgress(String progressMsg, long progress, long max) {
                            updateMessage(progressMsg);
                            updateProgress(progress, max);
                        }
                    });
                    task.init();
                    return task.call();
                }

                @Override
                protected void succeeded() {
                    super.succeeded();
                    updateMessage(task.getName() + " succeeded");
                }

                @Override
                protected void cancelled() {
                    super.cancelled();
                    task.setCancelled(true);
                    updateMessage(task.getName() + " cancelled");
                }
            };
        }
    }
}
